package org.eclipse.mylar.zest.layouts.algorithms;

import org.eclipse.mylar.zest.layouts.LayoutStyles;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalNode;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalRelationship;


/**
 * A simple algorithm to arrange graph nodes in a layered horizontal tree-like layout.
 * @see TreeLayoutAlgorithm
 *
 * @version  1.0
 * @author   Rob Lintern
 */
public class HorizontalTreeLayoutAlgorithm extends TreeLayoutAlgorithm {

	private double originalX;
	private double originalY;

	/**
	 * Creates a horizontal tree layout with no style
	 */
	public HorizontalTreeLayoutAlgorithm() {
		this( LayoutStyles.NONE );
	}

    /**
     *
     */
    public HorizontalTreeLayoutAlgorithm( int styles ) {
        super( styles );
    }

    protected void preLayoutAlgorithm(InternalNode[] entitiesToLayout, InternalRelationship[] relationshipsToConsider, double x, double y, double width, double height) {
    	// save the x and y, they will be used as an offset after the layout has run
    	this.originalX = x;
    	this.originalY = y;

    	// NOTE: we change the x, y to use 0, 0 to avoid rotating all nodes x, y after the layout has run
        // NOTE: width and height are swtiched here when calling super method
        super.preLayoutAlgorithm(entitiesToLayout, relationshipsToConsider, 0, 0, height, width);
    }

    protected void postLayoutAlgorithm(InternalNode[] entitiesToLayout, InternalRelationship[] relationshipsToConsider) {
        // swap x->y and width->height
        for (int i = 0; i < entitiesToLayout.length; i++) {
            InternalNode entity = entitiesToLayout[i];
            // add the original offset back into the position
			entity.setInternalLocation( originalX + entity.getInternalY(), originalY + entity.getInternalX() );
			entity.setInternalSize( entity.getInternalHeight(), entity.getInternalWidth() );
        }
    	super.postLayoutAlgorithm(entitiesToLayout, relationshipsToConsider);
    }

    protected boolean isValidConfiguration(boolean asynchronous, boolean continueous) {
		if ( asynchronous && continueous ) {
			return false;
		} else if ( asynchronous && !continueous ) {
			return true;
		} else if ( !asynchronous && continueous ) {
			return false;
		} else if ( !asynchronous && !continueous ) {
			return true;
		}

		return false;
    }

}
