/*
 * TouchGraph LLC. Apache-Style Software License
 *
 *
 * Copyright (c) 2001-2002 Alexander Shapiro. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgment:
 *       "This product includes software developed by
 *        TouchGraph LLC (http://www.touchgraph.com/)."
 *    Alternately, this acknowledgment may appear in the software itself,
 *    if and wherever such third-party acknowledgments normally appear.
 *
 * 4. The names "TouchGraph" or "TouchGraph LLC" must not be used to endorse
 *    or promote products derived from this software without prior written
 *    permission.  For written permission, please contact
 *    alex@touchgraph.com
 *
 * 5. Products derived from this software may not be called "TouchGraph",
 *    nor may "TouchGraph" appear in their name, without prior written
 *    permission of alex@touchgraph.com.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL TOUCHGRAPH OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
 * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ====================================================================
 *
 */

package org.eclipse.mylar.zest.layouts.algorithms;

import org.eclipse.mylar.zest.layouts.LayoutStyles;
import org.eclipse.mylar.zest.layouts.algorithms.internal.DynamicScreen;
import org.eclipse.mylar.zest.layouts.dataStructures.DisplayIndependentPoint;
import org.eclipse.mylar.zest.layouts.dataStructures.DisplayIndependentRectangle;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalNode;
import org.eclipse.mylar.zest.layouts.dataStructures.InternalRelationship;


/**
 * ********************************************************************
 * This is
 * the heart of the TouchGraph application. Please provide a Reference to
 * TouchGraph.com if you are influenced by what you see below. Your cooperation
 * will insure that this code remains opensource
 * ********************************************************************
 *
 * <p>
 * <b>Parts of this code build upon Sun's Graph Layout example.
 * http://java.sun.com/applets/jdk/1.1/demo/GraphLayout/Graph.java </b>
 * </p>
 *
 * @author Alexander Shapiro
 * @version 1.22-jre1.1 $Id: TGLayout.java,v 1.1 2002/09/19 15:58:08 ldornbusch
 *          Exp $
 */
public class TGLayoutAlgorithm extends ContinuousLayoutAlgorithm {


    private static final int DEFAULT_NODE_REPULSION = 100;
    private static final int DEFAULT_EDGE_LENGTH = 40;

    public static final long DEFAULT_TIME_OUT = 1000;

    private double damper = 0.0; // A low damper value causes the graph to move
                                 // slowly

    private double maxMotion = 0; // Keep an eye on the fastest moving node to
                                  // see if the graph is stabilizing

    private double lastMaxMotion = 0;

    private double motionRatio = 0; // It's sort of a ratio, equal to
                                    // lastMaxMotion/maxMotion-1

    private boolean damping = true; // When damping is true, the damper value
                                    // decreases

    private double rigidity = 1; // Rigidity has the same effect as the damper,
                                 // except that it's a constant

    private long timeout = DEFAULT_TIME_OUT;

	DynamicScreen TGDynamicScreen = null;

    public TGLayoutAlgorithm() {
    	this(LayoutStyles.NONE);
    }
    /**
     * Constructor with a supplied TGPanel <tt>tgp</tt>.
     */
    public TGLayoutAlgorithm( int styles ) {
        super( styles );
    }


	public void setLayoutArea(double x, double y, double width, double height) {

	}

	long startTime = 0;
	int iteration = 0;
	int maxIterations = 1000;

	/**
	 * Sets the timeout for the algorithm
	 * @param timeOut
	 */
	public void setTimeout( long timeOut ) {
		this.timeout = timeOut;
	}

	/**
	 * Gets the timeout
	 * @return
	 */
	public long getTimeout() {
		return this.timeout;
	}

	/**
	 * Scales the current iteration counter based on how long the algorithm has
	 * been running for.  You can set the MaxTime in maxTimeMS!
	 */
	private void setSprIterationsBasedOnTime() {
		if ( timeout <= 0 ) {
			return;
		}
		long currentTime = System.currentTimeMillis();
		double fractionComplete = ((currentTime - startTime) / (double) timeout);
		int currentIteration = (int) ( fractionComplete * maxIterations );
		if ( currentIteration > iteration )  {
			iteration = currentIteration;
		}


	}

	protected boolean performAnotherNonContinuousIteration() {
		setSprIterationsBasedOnTime();
		if (iteration <= maxIterations ) {
			return true;
		} else {
			return false;
		}
	}
	protected void preLayoutAlgorithm(InternalNode[] entitiesToLayout, InternalRelationship[] relationshipsToConsider, double x, double y, double width, double height) {
		TGDynamicScreen = new DynamicScreen((int)x,(int)y,(int)width,(int)height);
		for (int i = 0; i < entitiesToLayout.length; i++) {
            InternalNode node = entitiesToLayout[i];
			/*
			node.setInternalLocation( Math.random() * 2 -1, Math.random()*2 -1 );
			node.setDx(Math.random() * 2 - 1);
			node.setDy(Math.random()*2 -1 );
			*/
			setLocation( node,Math.random() * 2 - 1,  Math.random()*2 -1 );
			//node.setWidth(30);
			//node.setHeight(10);
		}

        damper = 1.0;
        damping = true;

        startTime = System.currentTimeMillis();
        iteration = 0;



	}

	protected void updateLayoutLocations() {
		// Do nothing, TG, already keeps things in the right place
		// TODO: I probably should do this though
	}

	protected void computeOneIteration(InternalNode[] entitiesToLayout, InternalRelationship[] relationshipsToConsider, double x, double y, double width, double height) {
		iteration++;
        relax(entitiesToLayout, relationshipsToConsider);
        //fireProgressEvent(1,1);
		try {
			// Delay between runs of the algorithm to let other threads run
			Thread.sleep(5);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
        defaultFitWithinBounds(entitiesToLayout, new DisplayIndependentRectangle(x,y,width,height) );
	}

    protected void postLayoutAlgorithm(InternalNode[] entitiesToLayout, InternalRelationship[] relationshipsToConsider) {
	    // do nothing

	}

	protected int getCurrentLayoutStep() {
		return iteration;
	}

	protected int getTotalNumberOfLayoutSteps() {
		return maxIterations;
	}



    //relaxEdges is more like tense edges up. All edges pull nodes closes together;
    private void relaxEdges(InternalRelationship [] relationshipsToConsider) {
        for (int i = 0; i < relationshipsToConsider.length; i++) {
            InternalRelationship relationship = relationshipsToConsider[i];
             double vx = getX(relationship.getDestination()) - getX(relationship.getSource());
            double vy = getY(relationship.getDestination()) - getY(relationship.getSource());
            double len = Math.sqrt(vx * vx + vy * vy);



            double dx = vx * rigidity; //rigidity makes edges tighter
            double dy = vy * rigidity;

            dx /= (DEFAULT_EDGE_LENGTH * 100);
            dy /= (DEFAULT_EDGE_LENGTH * 100);

            // Edges pull directly in proportion to the distance between the
            // nodes. This is good,
            // because we want the edges to be stretchy. The edges are ideal
            // rubberbands. They
            // They don't become springs when they are too short. That only
            // causes the graph to
            // oscillate.

            setDx(relationship.getDestination(), getDx(relationship.getDestination()) - (dx * len));
            setDy(relationship.getDestination(), getDy(relationship.getDestination()) - (dy * len));

            setDx(relationship.getSource(), getDx(relationship.getSource()) + (dx * len));
            setDy(relationship.getSource(), getDy(relationship.getSource()) + (dy * len));
       }
    }

    private void avoidLabels(InternalNode [] entitiesToLayout) {
        for (int i = 0; i < entitiesToLayout.length - 1; i++) {
            InternalNode n1 = entitiesToLayout[i];
            for (int j = i + 1; j < entitiesToLayout.length; j++) {
                InternalNode n2 = entitiesToLayout[j];
                double dx = 0;
                double dy = 0;
                double vx = getX(n1) - getX(n2);
                double vy = getY(n1) - getY(n2);
                double len = vx * vx + vy * vy; //so it's length squared
                if (len == 0) {
                    dx = Math.random(); //If two nodes are right on top of each
                                        // other, randomly separate
                    dy = Math.random();
                } else if (len < 600 * 600) { //600, because we don't want
                                              // deleted nodes to fly too far
                                              // away
                    dx = vx / len; // If it was sqrt(len) then a single node
                                   // surrounded by many others will
                    dy = vy / len; // always look like a circle. This might look
                                   // good at first, but I think
                    // it makes large graphs look ugly + it contributes to
                    // oscillation. A
                    // linear function does not fall off fast enough, so you get
                    // rough edges
                    // in the 'force field'
                }

                int repSum = DEFAULT_NODE_REPULSION * DEFAULT_NODE_REPULSION / 100;
                setDx(n1, getDx(n1) + dx * repSum * rigidity);
                setDy(n1, getDy(n1) + dy * repSum * rigidity);
                setDx(n2, getDx(n2) - dx * repSum * rigidity);
                setDy(n2, getDy(n2) - dy * repSum * rigidity);
            }
        }
    }

    public void damp() {
        if (damping) {
            if (motionRatio <= 0.001) { //This is important. Only damp when the
                                        // graph starts to move faster
                //When there is noise, you damp roughly half the time. (Which
                // is a lot)
                //
                //If things are slowing down, then you can let them do so on
                // their own,
                //without damping.

                //If max motion<0.2, damp away
                //If by the time the damper has ticked down to 0.9, maxMotion
                // is still>1, damp away
                //We never want the damper to be negative though
                if ((maxMotion < 0.2 || (maxMotion > 1 && damper < 0.9)) && damper > 0.01) {
					damper -= 0.01;
				} else if (maxMotion < 0.4 && damper > 0.003) {
					damper -= 0.003;
				} else if (damper > 0.0001) {
					damper -= 0.0001;
				}
            }
        }
        if (maxMotion < 0.001 && damping) {
            damper = 0;
        }
    }

    private void moveNodes(InternalNode [] entitiesToLayout) {
        lastMaxMotion = maxMotion;
        final double[] maxMotionA = new double[1];
        maxMotionA[0] = 0;

		TGDynamicScreen.cleanScreen();
		int counter = 0;
        for (int i = 0; i < entitiesToLayout.length; i++) {
            InternalNode n = entitiesToLayout[i];

			counter++;
			if ( counter == 100 ) {
				counter = 0;
				try {
					Thread.sleep(1);
				} catch (InterruptedException e) {
					e.printStackTrace();
				}
			}


            double dx = getDx(n);
            double dy = getDy(n);
            dx *= damper; //The damper slows things down. It cuts down jiggling
                          // at the last moment, and optimizes
            dy *= damper; //layout. As an experiment, get rid of the damper in
                          // these lines, and make a
            //long straight line of nodes. It wiggles too much and doesn't
            // straighten out.

            setDx(n, dx /2 ); //Slow down, but don't stop. Nodes in motion store
            			    	// momentum. This helps when the force
             setDy(n, dy /2 ); //on a node is very low, but you still want to get
             	              // optimal layout

            double distMoved = Math.sqrt(dx * dx + dy * dy); //how far did the
                                                             // node actually
                                                             // move?

			setLocation(n,getX(n) + Math.max(-30, Math.min(30, dx)),getY(n) + Math.max(-30, Math.min(30, dy))  );
            maxMotionA[0] = Math.max(distMoved, maxMotionA[0]);



         }

        maxMotion = maxMotionA[0];
        if (maxMotion > 0.4) {
			// subtract 1 to make a positive value mean that things are moving faster
            motionRatio = lastMaxMotion / maxMotion - 1;
		} else {
			//System.out.println("HEre");
            motionRatio = 0.4;
        }

        damp();

    }


    /*
     *
     */
    private void relax(InternalNode [] entitiesToLayout, InternalRelationship [] relationshipsToConsider) {
        relaxEdges(relationshipsToConsider);
        avoidLabels(entitiesToLayout);
        moveNodes(entitiesToLayout);
    }

	private void setLocation( InternalNode entity, double x, double y ) {
		TGDynamicScreen.removeNode( entity );
		entity.setInternalLocation( x, y );
		TGDynamicScreen.addNode( entity );
		DisplayIndependentPoint p = TGDynamicScreen.getScreenLocation( entity );
		if ( Math.abs( entity.getCurrentX() - p.x ) < 3 &&
			 Math.abs( entity.getCurrentX() - p.y ) < 3 )   {
		}
		else {
			entity.setLocation( p.x, p.y );
		}
	}

    private void setDx (InternalNode entity, double dx) {
		entity.setDx( dx );
    }
    private void setDy (InternalNode entity, double dy) {
		entity.setDy( dy );
    }

    private double getX (InternalNode entity) {
		if ( entity.hasPreferredLocation() ) {

			DisplayIndependentPoint point = TGDynamicScreen.getVirtualLocation(new DisplayIndependentPoint( entity.getPreferredX(), entity.getPreferredY() ) );
			setDx(entity, Math.abs( point.x - entity.getInternalX()) );
			return point.x;
		}
		return entity.getInternalX();
    }
    private double getY (InternalNode entity) {
		if ( entity.hasPreferredLocation() ) {
			DisplayIndependentPoint point = TGDynamicScreen.getVirtualLocation(new DisplayIndependentPoint( entity.getPreferredX(), entity.getPreferredY() ) );
            setDy(entity, Math.abs( point.y - entity.getInternalY()) );
			return point.y;
		}
		return entity.getInternalY();
    }
    private double getDx (InternalNode entity) {
		return entity.getDx();
    }
    private double getDy (InternalNode entity) {
		return entity.getDy();
    }

    protected boolean isValidConfiguration(boolean asynchronous, boolean continueous) {
		if ( asynchronous && continueous ) {
			return true;
		} else if ( asynchronous && !continueous ) {
			return true;
		} else if ( !asynchronous && continueous ) {
			return false;
		} else if ( !asynchronous && !continueous ) {
			return true;
		}

		return false;
	}



} // end com.touchgraph.graphlayout.TGLayout
